package ROTool::RO_GPS::Client;
use strict;
use ROTool;
use ROTool::Client;
use IO::Socket::INET;
use vars qw(@ISA $VERSION);

@ISA = qw( ROTool::Client );

my $SERVER_HEADER = "RO-GPS/S04";
my $CLIENT_HEADER = "RO-GPS/C04";
my $APPRI_VER     = "RO_GPS.pm.a100";

sub import{
    &ROTool::typechecker_regist(\&typechecker);
}

sub typechecker{
    #not a method.
    my $self = shift;
    if($self->{"buf"} =~ /^$CLIENT_HEADER\001/){
	bless $self;
	$self->new();
	return 1;
    }else{
	return 0;
    }
}

sub is_client{
    return 1;
}

sub is_server{
    return 0;
}

sub recv{
    my($self) = shift;

    my($ret) = $self->SUPER::recv();
    &ROTool::printlog(2," [$self->{oid}] parse message\n");
    while($self->{"buf"} =~ s/^(.*?)\002//){
	my $msg = $1;
	@_ = split(/\001/,$msg);
	splice(@_,0,1);

	my($message) = $_[10];
	$message =~ s/^"(.*)"$/$1/;
	if($message ne $self->{"param"}->{"message"}){
	    $self->{"param"}->{"time"} =
		sprintf("%02d:%02d", (localtime())[2,1]);
	}

	@{$self->{"param"}}{"APPRI_VER","id","server","name","area",
			    "hp","hpmax","sp","spmax","lv","message"} = @_;
	$self->{"param"}->{"id"}       =~ s/^"(.*)"$/$1/;
	$self->{"param"}->{"name"}     =~ s/^"(.*)"$/$1/;
	$self->{"param"}->{"area"}     =~ s/^"(.*)"$/$1/;
	$self->{"param"}->{"message"}  =~ s/^"(.*)"$/$1/;
	$self->{"param"}->{"lv"}       =~ s{^"((.*?)/(.*?)/(.*?))"}{$1};
	@{$self->{"param"}}{"class","baselv","joblv"} = ($2,$3,$4);
    }
    if(&ROTool::UA_regist($self->{"param"}->{"id"},$self) == 0){
	return 0;
    }
    return $ret;
}

sub send{
    my $self = shift;

    my @msg;
    my $count = 0;
    foreach my $client (&ROTool::UA_list()){
	next if($client->is_client() == 0);
	$count++;
	my %param = %{$client->{param}};
	$param{"id"}      = qq{"$param{id}"};
	$param{"server"}||= "----";
	$param{"name"}    = qq{"$param{name}"};
	$param{"area"}    = qq{"$param{area}"};
	$param{"hp"}    ||= 0;
	$param{"hpmax"} ||= 0;
	$param{"sp"}    ||= 0;
	$param{"spmax"} ||= 0;
	$param{"class"} ||= "--";
	$param{"baselv"}||= "--";
	$param{"joblv"} ||= "--";
	$param{"lv"}      = qq{"$param{class}/$param{baselv}/$param{joblv}"};
	$param{"message"} = qq{"$param{message}"};
	push(@msg,@param{"APPRI_VER","id","server","name","area",
			"hp","hpmax","sp","spmax","lv","message"});
    }
    unshift(@msg,($SERVER_HEADER,$count));
    
    my $sock = $self->{"sock"};
    my $msg = join("\001",@msg)."\002";
    $sock->syswrite($msg,length($msg));
    &ROTool::printlog(2,$msg."\n");
}

sub is_keep_message{
    my $self = shift;
    return 0 if($self->{"Message_lasting"});
    return 1 if($self->{"param"}->{"message"}
		=~ /^(\(|\x81\x69).*?(\)|\x81\x6a)$/);
    return 0;
}

1;
